/* ---------------------------------------------------------
 *  MCP23008.c -                  
 * ---------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <linux/i2c-dev.h>



//#define MCP23008_ADDR		0x20		//A2=0 A1=0 A0=0
#define MCP23008_ADDR		0x21		//A2=0 A1=0 A0=1
//#define MCP23008_ADDR		0x22		//A2=0 A1=1 A0=0
//#define MCP23008_ADDR		0x23		//A2=0 A1=1 A0=1

#define MCP23008_DIR		0
#define MCP23008_POL		1
#define MCP23008_CON		5
#define MCP23008_GPIO		9

//Limit is an input on IO0
//Enable is active low on IO1
//USM0-4 are on IO1-5 00=full 01=half 10=quarter 11=1/8
#define Full_step		(0<<5 | 0<<4)
#define Half_step		(0<<5 | 1<<4)
#define Quarter_step		(1<<5 | 0<<4)
#define Eight_step		(1<<5 | 1<<4)


#define Limit			(1<<7)
#define Enable			(1<<6)
#define USM1			(1<<5)
#define USM0			(1<<4)
#define DAC2			(1<<2)
#define DAC1			(1<<1)
#define DAC0			(1<<0)

#define Current_000		(0<<2 | 0<<1 | 0<<0)
#define Current_125		(0<<2 | 0<<1 | 1<<0)
#define Current_250		(0<<2 | 1<<1 | 0<<0)
#define Current_375		(0<<2 | 1<<1 | 1<<0)
#define Current_625		(1<<2 | 0<<1 | 0<<0)
#define Current_750		(1<<2 | 0<<1 | 1<<0)
#define Current_875		(1<<2 | 1<<1 | 0<<0)
#define Current_1000		(1<<2 | 1<<1 | 1<<0)

int file;

void MCP23008_write(unsigned char reg, unsigned char value){
    unsigned char data[2];
    data[0] = reg;
    data[1] = value;
    if ( write(file,data,2) != 2) {
        printf("%d ",write(file,data,2));
        printf("MCP23008_write failed\n");
        exit(1);
    }
}

unsigned char MCP23008_read(unsigned char reg){
    unsigned char data[1];
    unsigned char result[1];
    data[0] = reg;
    if ( write(file,data,1) != 1) {
        printf("MCP23008_read failed\n");
        exit(1);
    }
    read(file,result,1);
    return result[0];
}

void scan(void){
	unsigned int addr;
    	unsigned char data[1];
    	unsigned char result[1];
	
	printf("Scanning for I2C devices. \n");
    	for(addr = 0;addr <0x80; addr++){
		if ((ioctl(file,I2C_SLAVE,addr)) < 0){
          		printf("ioctl failed, ");
            		printf("addr=0x%02x\n",addr);
            		exit(1);
    		} else {
			data[0] = addr;
			printf(".");
			if(write(file,data,1) == 1){
    				read(file,result,1);
	    			printf("\nI2C slave device found at address 0x%02x \n", addr);
			}
    		}
	}
	printf("\n");
}


unsigned char MCP23008(unsigned char reg){
    unsigned char data[1];
    unsigned char result[1];
    data[0] = reg;
    if ( write(file,data,1) != 1) {
        printf("MCP23008_read failed\n");
	return result[0];
    }
    read(file,result,1);
    return result[0];
}


int main(int argc, char *argv[]) {
    unsigned long addr;
    int rc;

    printf("MCP23008 test\n");

    if ((file = open("/dev/i2c-0",O_RDWR)) < 0) {
        printf("%s: ",argv[0]);
        printf("failed to open /dev/i2c-0, err=%d\n",file);
        exit(1);
    }

	scan();

    addr = MCP23008_ADDR; // The I2C address 
    if ((rc=ioctl(file,I2C_SLAVE,addr)) < 0) {
        printf("%s: addr ioctl failed, ",argv[0]);
        printf("addr=0x%03lX, err=%d\n",addr,rc);
        exit(1);
    } else {
        printf("%s: ioctl set ",argv[0]);
        printf("I2C_SLAVE addr=0x%03lX\n",addr);
	MCP23008_write(MCP23008_DIR, Limit);//all output except limit (low for out)
	printf("Direction: 0x%2x \n", MCP23008_read(MCP23008_DIR));
	MCP23008_write(MCP23008_GPIO, Current_375);//Vref
	printf("Vref:0x%2x \n", MCP23008_read(MCP23008_GPIO));

	printf("Input:0x%2x ", MCP23008_read(MCP23008_GPIO)); 
	MCP23008_write(MCP23008_DIR, Limit);//Limit input 
  	MCP23008_write(MCP23008_GPIO, Full_step);
	printf("Output:0x%2x\n", MCP23008_read(MCP23008_GPIO)); 
    }
return(0);
}
//while(1) {}

 /*       while(1) {
	        printf("Input:0x%2x ", MCP23008_read(MCP23008_GPIO)); 
	        MCP23008_write(MCP23008_DIR, Limit);//Limit input 
	        MCP23008_write(MCP23008_GPIO, Full_step);
	        printf("Output:0x%2x\r", MCP23008_read(MCP23008_GPIO)); 
        }
    }*/




