/* ---------------------------------------------------------
 *  set_step_mode.c -                  
 * ---------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <linux/i2c-dev.h>


#define XAXIS		0x20		//A2=0 A1=0 A0=0
#define YAXIS		0x21		//A2=0 A1=0 A0=1
#define ZAXIS		0x22		//A2=0 A1=1 A0=0
#define other		0x23		//A2=0 A1=1 A0=1
//#define MCP23008_ADDR		0x20		//A2=0 A1=0 A0=0
//#define MCP23008_ADDR		0x21		//A2=0 A1=0 A0=1
//#define MCP23008_ADDR		0x22		//A2=0 A1=1 A0=0
//#define MCP23008_ADDR		0x23		//A2=0 A1=1 A0=1

#define MCP23008_DIR		0
#define MCP23008_GPIO		9

//Limit is an input on IO0
#define Limit			(1<<7)


#define Current_00		(0<<2 | 0<<1 | 0<<0)
#define Current_10		(1<<2 | 0<<1 | 0<<0)
#define Current_20		(1<<2 | 0<<1 | 0<<0)
#define Current_30		(0<<2 | 1<<1 | 0<<0)
#define Current_40		(0<<2 | 1<<1 | 0<<0)
#define Current_50		(0<<2 | 0<<1 | 1<<0)
#define Current_60		(0<<2 | 0<<1 | 1<<0)
#define Current_70		(1<<2 | 0<<1 | 1<<0)
#define Current_80		(1<<2 | 0<<1 | 1<<0)
#define Current_90		(0<<2 | 1<<1 | 1<<0)
#define Current_100		(1<<2 | 1<<1 | 1<<0)

#define Direction_00	(1<<2 | 1<<1 | 1<<0)
#define Direction_10	(0<<2 | 0<<1 | 0<<0)
#define Direction_20	(0<<2 | 0<<1 | 1<<0)
#define Direction_30	(0<<2 | 0<<1 | 0<<0)
#define Direction_40	(0<<2 | 0<<1 | 1<<0)
#define Direction_50	(0<<2 | 0<<1 | 0<<0)
#define Direction_60	(0<<2 | 1<<1 | 0<<0)
#define Direction_70	(0<<2 | 0<<1 | 0<<0)
#define Direction_80	(0<<2 | 1<<1 | 0<<0)
#define Direction_90	(1<<2 | 0<<1 | 0<<0)
#define Direction_100	(0<<2 | 0<<1 | 0<<0)

int file;

void MCP23008_write(unsigned char reg, unsigned char value){
    unsigned char data[2];
    data[0] = reg;
    data[1] = value;
    if ( write(file,data,2) != 2) {
        printf("%d ",write(file,data,2));
        printf("MCP23008_write failed\n");
        exit(1);
    }
}

unsigned char MCP23008_read(unsigned char reg){
    unsigned char data[1];
    unsigned char result[1];
    data[0] = reg;
    if ( write(file,data,1) != 1) {
        printf("MCP23008_read failed\n");
        exit(1);
    }
    read(file,result,1);
    return result[0];
}

unsigned int scan(void){
	unsigned int addr;
    unsigned char data[1];
    unsigned char result[1];
	unsigned int rc = -1;
	
    	for(addr = 0;addr <0x80; addr++){
		if ((ioctl(file,I2C_SLAVE,addr)) < 0){
          		printf("ioctl failed, ");
            	printf("addr=0x%02x\n",addr);
            	exit(1);
    	} else {
			data[0] = addr;
			if(write(file,data,1) == 1){
    			read(file,result,1);
//	    		printf("\nI2C slave device found at address 0x%02x \n", addr);
				rc = addr;
			}
    	}
	}
	return rc;
}


unsigned char MCP23008(unsigned char reg){
    unsigned char data[1];
    unsigned char result[1];
    data[0] = reg;
    if ( write(file,data,1) != 1) {
        printf("MCP23008_read failed\n");
		return result[0];
    }
    read(file,result,1);
    return result[0];
}

void Disable_motor(void){
    unsigned char temp;
 	temp = MCP23008_read(MCP23008_GPIO);
	temp |= (1<<6);			//motor disable
	MCP23008_write(MCP23008_GPIO,temp);
}

void Enable_motor(void){
    unsigned char temp;
 	temp = MCP23008_read(MCP23008_GPIO);
	temp &= ~(1<<6);		//motor enable
	MCP23008_write(MCP23008_GPIO,temp);
}

void map_percentage_current(unsigned char percentage){
unsigned char current,current_state,direction,direction_state;
	current_state = MCP23008_read(MCP23008_GPIO) & 0xf8;
	direction_state = MCP23008_read(MCP23008_DIR) & 0xf8;
	if( percentage == 0){
		current = Current_00;
		direction = Direction_00;
	};
    if(percentage == 10){
		current = Current_10;
		direction = Direction_10;
	};
    if(percentage == 20){
		current = Current_20;
		direction = Direction_20;
	};
    if(percentage == 30){
		current = Current_30;
		direction = Direction_30;
	};
    if(percentage == 40){
		current = Current_40;
		direction = Direction_40;
	};
    if(percentage == 50){
		current = Current_50;
		direction = Direction_50;
	};
    if(percentage == 60){
		current = Current_60;
		direction = Direction_60;
	};
    if(percentage == 70){
		current = Current_70;
		direction = Direction_70;
	};
    if(percentage == 80){
		current = Current_80;
		direction = Direction_80;
	};
    if(percentage == 90){
		current = Current_90;
		direction = Direction_90;
	};
    if(percentage == 100){
		current = Current_100;
		direction = Direction_100;
	};
	direction = direction & 0x07;
	direction = direction | direction_state;
    MCP23008_write(MCP23008_DIR, direction);

	current = current & 0x07;
	current = current | current_state;
    MCP23008_write(MCP23008_GPIO, current);
//    printf("current:0x%2x\n", MCP23008_read(MCP23008_GPIO));
//    printf("direction:0x%2x\n", MCP23008_read(MCP23008_DIR));
}

int main(int argc, char *argv[]) {
    unsigned long addr;
    int rc;

    if(argc !=3){
		printf("Insufficient parameters.\n");
		printf("Please specify the motor (X,Y,Z,other) ");
		printf("and the current (0..100).\n");
    }

    if ((file = open("/dev/i2c-0",O_RDWR)) < 0) {
        printf("%s: ",argv[0]);
        printf("failed to open /dev/i2c-0, err=%d\n",file);
        exit(1);
    }

    addr = scan();

    if ((rc=ioctl(file,I2C_SLAVE,addr)) < 0) {
        printf("%s: addr ioctl failed, ",argv[0]);
        printf("addr=0x%03lX, err=%d\n",addr,rc);
        exit(1);
    } else {
    MCP23008_write(MCP23008_DIR, Limit);//Limit input 
    switch (*argv[1]){
    	case 'X':
    	case 'x':
			if(addr != XAXIS){
				printf("Can't find X axis.\n");
			}else{
				map_percentage_current(atoi(argv[2]));
//				printf("X axis output:0x%2x\n", MCP23008_read(MCP23008_GPIO)); 
			}
 	  		break;

  	  	case 'Y':
  	  	case 'y':
			if(addr != YAXIS){
				printf("Can't find Y axis.\n");
			}else{
				map_percentage_current(atoi(argv[2]));
//				printf("Y axis output:0x%2x\n", MCP23008_read(MCP23008_GPIO)); 
			}
	   	break;

 	   	case 'Z':
 	   	case 'z':
			if(addr != ZAXIS){
				printf("Can't find Z axis.\n");
			}else{
				map_percentage_current(atoi(argv[2]));
//				printf("Z axis output:0x%2x\n", MCP23008_read(MCP23008_GPIO)); 
			}
  	 	break;

   	 	default:
			if(addr != other){
				printf("Can't find other axis.\n");
			}else{
				map_percentage_current(atoi(argv[2]));
//				printf("Other axis output:0x%2x\n", MCP23008_read(MCP23008_GPIO)); 
			}
   	 		break;
    	}
    }
	return(0);
}





