/* ---------------------------------------------------------
 *  set_step_mode.c -                  
 * ---------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <linux/i2c-dev.h>


#define XAXIS		0x20		//A2=0 A1=0 A0=0
#define YAXIS		0x21		//A2=0 A1=0 A0=1
#define ZAXIS		0x22		//A2=0 A1=1 A0=0
#define other		0x23		//A2=0 A1=1 A0=1
//#define MCP23008_ADDR		0x20		//A2=0 A1=0 A0=0
//#define MCP23008_ADDR		0x21		//A2=0 A1=0 A0=1
//#define MCP23008_ADDR		0x22		//A2=0 A1=1 A0=0
//#define MCP23008_ADDR		0x23		//A2=0 A1=1 A0=1

#define MCP23008_DIR		0
#define MCP23008_GPIO		9

//Limit is an input on IO0
#define Limit			(1<<7)


#define Current_000		(0<<2 | 0<<1 | 0<<0)
#define Current_125		(0<<2 | 0<<1 | 1<<0)
#define Current_250		(0<<2 | 1<<1 | 0<<0)
#define Current_375		(0<<2 | 1<<1 | 1<<0)
#define Current_625		(1<<2 | 0<<1 | 0<<0)
#define Current_750		(1<<2 | 0<<1 | 1<<0)
#define Current_875		(1<<2 | 1<<1 | 0<<0)
#define Current_1000		(1<<2 | 1<<1 | 1<<0)

int file;

void MCP23008_write(unsigned char reg, unsigned char value){
    unsigned char data[2];
    data[0] = reg;
    data[1] = value;
    if ( write(file,data,2) != 2) {
        printf("%d ",write(file,data,2));
        printf("MCP23008_write failed\n");
        exit(1);
    }
}

unsigned char MCP23008_read(unsigned char reg){
    unsigned char data[1];
    unsigned char result[1];
    data[0] = reg;
    if ( write(file,data,1) != 1) {
        printf("MCP23008_read failed\n");
        exit(1);
    }
    read(file,result,1);
    return result[0];
}

unsigned int scan(void){
	unsigned int addr;
    unsigned char data[1];
    unsigned char result[1];
	unsigned int rc = -1;
	
    	for(addr = 0;addr <0x80; addr++){
		if ((ioctl(file,I2C_SLAVE,addr)) < 0){
          		printf("ioctl failed, ");
            	printf("addr=0x%02x\n",addr);
            	exit(1);
    	} else {
			data[0] = addr;
			if(write(file,data,1) == 1){
    			read(file,result,1);
//	    		printf("\nI2C slave device found at address 0x%02x \n", addr);
				rc = addr;
			}
    	}
	}
	return rc;
}


unsigned char MCP23008(unsigned char reg){
    unsigned char data[1];
    unsigned char result[1];
    data[0] = reg;
    if ( write(file,data,1) != 1) {
        printf("MCP23008_read failed\n");
		return result[0];
    }
    read(file,result,1);
    return result[0];
}

void map_percentage_current(unsigned char percentage)
{
unsigned char current,current_state;
	current_state = MCP23008_read(MCP23008_GPIO) & 0xf8;
	if( percentage == 0){current = Current_000;};
    if((0 < percentage) && (percentage<= 15)){current = Current_125;};
    if((15 < percentage) && (percentage <= 30)){current = Current_250;};
    if((30 < percentage) && (percentage <= 45)){current = Current_375;};
    if((45 < percentage) && (percentage <= 60)){current = Current_625;};
    if((60 < percentage) && (percentage <= 75)){current = Current_750;};
    if((75 < percentage) && (percentage <= 90)){current = Current_875;};
    if((90 < percentage) && (percentage <= 100)){current = Current_1000;};
	current = current & 0x07;
	current = current | current_state;
    MCP23008_write(MCP23008_GPIO, current);
    printf("current:0x%2x\n", MCP23008_read(MCP23008_GPIO));
}

int main(int argc, char *argv[]) {
    unsigned long addr;
    int rc;

    if(argc !=3){
		printf("Insufficient parameters.\n");
		printf("Please specify the motor (X,Y,Z,other) ");
		printf("and the current (0..100).\n");
    }

    if ((file = open("/dev/i2c-0",O_RDWR)) < 0) {
        printf("%s: ",argv[0]);
        printf("failed to open /dev/i2c-0, err=%d\n",file);
        exit(1);
    }

    addr = scan();

    if ((rc=ioctl(file,I2C_SLAVE,addr)) < 0) {
        printf("%s: addr ioctl failed, ",argv[0]);
        printf("addr=0x%03lX, err=%d\n",addr,rc);
        exit(1);
    } else {
    MCP23008_write(MCP23008_DIR, Limit);//Limit input 
    switch (*argv[1]){
    	case 'X':
    	case 'x':
			if(addr != XAXIS){
				printf("Can't find X axis.\n");
			}else{
				map_percentage_current(atoi(argv[2]));
//				printf("X axis output:0x%2x\n", MCP23008_read(MCP23008_GPIO)); 
			}
 	  		break;

  	  	case 'Y':
  	  	case 'y':
			if(addr != YAXIS){
				printf("Can't find Y axis.\n");
			}else{
				map_percentage_current(atoi(argv[2]));
//				printf("Y axis output:0x%2x\n", MCP23008_read(MCP23008_GPIO)); 
			}
	   	break;

 	   	case 'Z':
 	   	case 'z':
			if(addr != ZAXIS){
				printf("Can't find Z axis.\n");
			}else{
				map_percentage_current(atoi(argv[2]));
//				printf("Z axis output:0x%2x\n", MCP23008_read(MCP23008_GPIO)); 
			}
  	 	break;

   	 	default:
			if(addr != other){
				printf("Can't find other axis.\n");
			}else{
				map_percentage_current(atoi(argv[2]));
//				printf("Other axis output:0x%2x\n", MCP23008_read(MCP23008_GPIO)); 
			}
   	 		break;
    	}
    }
	return(0);
}





