/* ---------------------------------------------------------
 *  PCA9536.c -                  
 * ---------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>
#include <linux/i2c-dev.h>


#define I2C_SLAVE		0x0703   // Change slave address 

#define PCA9536_ADDR		0x41
#define PCA9536_IN		0
#define PCA9536_OUT		1
#define PCA9536_POL		2
#define PCA9536_DIR		3

//Limit is an input on IO0
//Enable is active low on IO1
//USM0-1 are on IO1-2 00=full 01=half 10=quarter 11=1/8
#define Full_step		0xF0
#define Half_step		0xF4
#define Quarter_step		0xF8
#define Eight_step		0xFC


int file;

void PCA9536_write(unsigned char reg, unsigned char value){
    unsigned char data[2];
    data[0] = reg;
    data[1] = value;
    if ( write(file,data,2) != 2) {
        printf("%d ",write(file,data,2));
        printf("PCA9536_write write failed\n");
        exit(1);
    }
}

unsigned char PCA9536_read(unsigned char reg){
    unsigned char data[1];
    unsigned char result[1];
    data[0] = reg;
    if ( write(file,data,1) != 1) {
        printf("PCA9536_read write failed\n");
        exit(1);
    }
    read(file,result,1);
    return result[0];
}


int main(int argc, char *argv[]) {
    unsigned long addr;
    int rc;

    printf("PCA9536 test\n");

    if ((file = open("/dev/i2c-0",O_RDWR)) < 0) {
        printf("%s: ",argv[0]);
        printf("failed to open /dev/i2c-0, err=%d\n",file);
        exit(1);
    }

    addr = PCA9536_ADDR; // The I2C address 
    if ((rc=ioctl(file,I2C_SLAVE,addr)) < 0) {
        printf("%s: addr ioctl failed, ",argv[0]);
        printf("addr=0x%03lX, err=%d\n",addr,rc);
        exit(1);
    } else {
        printf("%s: ioctl set ",argv[0]);
        printf("I2C_SLAVE addr=0x%03lX\n",addr);

        while(1) {
	        printf("Input:0x%2x ", PCA9536_read(PCA9536_IN)); 
	        PCA9536_write(PCA9536_DIR, 0xf1);//IO0 input IO1-3 output
	        PCA9536_write(PCA9536_OUT, Full_step);
	        printf("Output:0x%2x\r", PCA9536_read(PCA9536_OUT)); 
        }
    }
    return(0);
}



