#ifndef __LIBGPIO_H__
#define __LIBGPIO_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/types.h>
#include <stdint.h>

// __ Constants ____________________________
#define MMAP_DEVICE "/dev/mem"
#define GPIO_BASE_ADDRESS  0x10000000
#define GPIO_BASE_SIZE  0xFFF
#define GPIO_21_0_DATA_OFFSET  	0x620
#define GPIO_21_0_DIR_OFFSET  	0x624
#define GPIO_21_0_TOG_OFFSET  	0x634
#define GPIO_27_22_DATA_OFFSET  0x670
#define GPIO_27_22_DIR_OFFSET  	0x674

#define GPIO_MODE_OFFSET 0x60
#define I2C_MODE	1<<0
#define SPI_MODE  	1<<1
#define UARTF_MODE	7<<2
#define UARTL_MODE	1<<5
#define JTAG_MODE	1<<6
#define PHYBT_MODE	2<<14
#define CS_MODE		2<<22

//  GPIO bitmasks
#define GPIO0  (1<<0)
#define GPIO7  (1<<7)
#define GPIO8  (1<<8)
#define GPIO9  (1<<9)
#define GPIO14 (1<<14)
#define GPIO25 (1<<3)
#define GPIO26 (1<<4)

// __ Registers ____________________________

//Direct access to the GPIO data and dir registers
extern volatile uint32_t *PORT0_21;
extern volatile uint32_t *DIR0_21;
extern volatile uint32_t *TOGGGLE0_21;
extern volatile uint32_t *PORT22_27;
extern volatile uint32_t *DIR22_27;

// __ API __________________________________
void		gpio_init();
void		gpio_dispose();

void		gpio_set_mode(unsigned int mode);
uint32_t 	gpio_get_mode();

void		gpio_setdir(uint32_t value);
uint32_t	gpio_getdir();

void 		gpio_set(uint32_t value);
void 		gpio_toggle(uint32_t value);
uint32_t 	gpio_get();

#ifdef __cplusplus
}
# endif

#endif //__LIBGPIO_H__
