#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "libgpio.h"

static inline void send_bit(uint8_t on)
{
  int i;
  if (on){
      // repeat bit high for 0.7us
      for (i = 11; i > 0; i--) gpio_set(GPIO0);
      // repeat bit low for 0.6us
      for (i = 9; i > 0; i--) gpio_set(0);//clear
  }else{
      // repeat bit high for 0.35us
      for (i = 5; i > 0; i--) gpio_set(GPIO0);
      // repeat bit low for 0.8us
      for (i = 12; i > 0; i--) gpio_set(0);
  }
}


void send_leds(uint8_t *buffer, uint16_t numleds)
{
  int16_t i = numleds*3;
  gpio_setdir(GPIO0);
  for (; i >=0; i--){
      uint8_t val =  *buffer++;

      uint8_t bit;
      for (bit = 0; bit < 8; bit++){
	  send_bit(val & 0b10000000);	// 25sec red roundtrip
	  val <<= 1;
      }
  }
  usleep(50);
}

//To run the tests, set the scope probes on GPIOs 7
void test_gpio_set()
{
	int i;

	for (i = 0; i < 10000000; ++i){
		gpio_toggle(GPIO7);
//		gpio_set(GPIO7);
	}
}

int main(int argc, char *argv[])
{
	// Initialize GPIO library. Needed before any GPIO pins are used. 
	gpio_init();
	
	printf("Direction  0 to 21 = 0x%x\r\n",*DIR0_21);
	printf("Port       0 to 21 = 0x%x\r\n",*PORT0_21);
	printf("Direction 22 to 27 = 0x%x\r\n",*DIR22_27);
	printf("Port      22 to 27 = 0x%x\r\n",*PORT22_27);
	printf("MODE               = 0x%x\r\n",gpio_get_mode());


	int mode = gpio_get_mode();
	if(mode & I2C_MODE){
		printf("GPIO mode (GPIO1-2)\r\n");
	}else{
		printf("I2C mode (SDA(1),SCL(2))\r\n");
	}
	if(mode & SPI_MODE){
		printf("GPIO mode (GPIO3-6)\r\n");
	}else{
		printf("SPI mode (CS(3),SCK(4),MOSI(5),MISO(6))\r\n");
	}
	if(mode & (UARTF_MODE&(7<<2))){
		printf("GPIO mode (GPIO7-14)\r\n");
	}else if(mode & (UARTF_MODE&(6<<2))){
		printf("I2S GPIO mode (SCLK,WS,SDO,SDI,GPIO11-14)\r\n");
	}else if(mode & (UARTF_MODE&(5<<2))){
		printf("UARTF GPIO mode (RTS,TXD,CTS,RXD,GPIO11-14)\r\n");
	}else if(mode & (UARTF_MODE&(4<<2))){
		printf("GPIO PCM mode (GPIO7-10,FS,CLK,DRX,DTX)\r\n");
	}else if(mode & (UARTF_MODE&(3<<2))){
		printf("I2S UARTF mode (SCLK,WS,SDO,SDI,RTS,TXD,CTS,RXD)\r\n");
	}else if(mode & (UARTF_MODE&(2<<2))){
		printf("I2S PCM mode (SCLK,WS,SDO,SDI,FS,CLK,DRX,DTX))\r\n");
	}else if(mode & (UARTF_MODE&(1<<2))){
		printf("UARTF PCM mode (RTS,TXD,CTS,RXD,FS,CLK,DRX,DTX)\r\n");
	}else{
		printf("UARTF mode (RTS,TXD,CTS,RXD,DTR,DCD,DSR,RIN)\r\n");
	}
	if(mode & UARTL_MODE){
		printf("UARTL mode (GPIO15-16)\r\n");
	}else{
		printf("UARTL mode (TXD2(15),RXD2(16))\r\n");
	}
	if(mode & JTAG_MODE){
		printf("JTAG mode (GPIO17-21)\r\n");
	}else{
		printf("JTAG mode (TDO(17),TDI(18),TMS(19),TRST(20))\r\n");
	}
	if(mode & (PHYBT_MODE&(2<<15))){	
		printf("Bluetooth mode (22-26)\r\n");
	}else if(mode & (PHYBT_MODE&(1<<15))){
		printf("GPIO mode (GPIO22-26)\r\n");
	}else{
		printf("EPHY LED0-4 (22-26)\r\n");
	}

	if(mode & (CS_MODE&(2<<22))){
		printf("GPIO mode (GPIO27)\r\n");
	}else if(mode & (CS_MODE&(1<<22))){
		printf("Watchdog reset output (27)\r\n");
	}else{
		printf("CS1 mode (27)\r\n");
	}


	// Set I2C and SPI shared pins to GPIO.
	// This makes 6 more GPIOs available to us (1,2 -> I2C, 3,4,5,6 -> SPI)
//	gpio_set_mode(SPI_MODE_GPIO | I2C_MODE_GPIO);
	uint8_t RGB[] = {0xff,0x00,0x00};
	uint8_t *p = RGB;
	int i,j;
	for(j=0;j<100;j++){
	for(i=0;i<2000;i++){
		send_leds(p, 1);
		usleep(50);
	}
	uint8_t RGB1[] = {0x00,0xff,0x00};
	uint8_t *p1 = RGB1;
	for(i=0;i<2000;i++){
		send_leds(p1, 1);
		usleep(50);
	}
	uint8_t RGB2[] = {0x00,0x00,0xff};
	uint8_t *p2 = RGB2;
	for(i=0;i<2000;i++){
		send_leds(p2, 1);
		usleep(50);
	}
}
	// Set direction of all GPIOs to OUTPUT
//	gpio_setdir(GPIO7);  
//	test_gpio_set();


	
	// Cleanup GPIO library
	gpio_dispose();

	return 0;
}
