#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <termios.h>

int set_interface_attribs (int fd, int speed, int parity){
        struct termios tty;
        memset (&tty, 0, sizeof tty);
        if (tcgetattr (fd, &tty) != 0)
        {
			printf("error %d from tcgetattr", errno);
            return -1;
        }

        cfsetospeed (&tty, speed);
        cfsetispeed (&tty, speed);

        tty.c_cflag = (tty.c_cflag & ~CSIZE) | CS8;// 8-bit 
        tty.c_iflag &= ~IGNBRK;         // disable 
        tty.c_lflag = 0;                // no echo
        tty.c_oflag = 0;                // no remap or delay
        tty.c_cc[VMIN]  = 0;            // read doesn't block
        tty.c_cc[VTIME] = 5;            // 0.5 sec timeout

        tty.c_iflag &= ~(IXON | IXOFF | IXANY); //disable xon

        tty.c_cflag |= (CLOCAL | CREAD);// ignore modem 
                                        // enable reading
        tty.c_cflag &= ~(PARENB | PARODD); // no parity
        tty.c_cflag |= parity;
        tty.c_cflag &= ~CSTOPB;
        tty.c_cflag &= ~CRTSCTS;

        if (tcsetattr (fd, TCSANOW, &tty) != 0)
        {
        	printf("error %d from tcsetattr", errno);
            return -1;
        }
        return 0;
}

void set_blocking (int fd, int should_block){
        struct termios tty;
        memset (&tty, 0, sizeof tty);
        if (tcgetattr (fd, &tty) != 0){
        	printf("error %d from tggetattr", errno);
            return;
        }

        tty.c_cc[VMIN]  = should_block ? 1 : 0;
        tty.c_cc[VTIME] = 5;            	// 0.5 seconds read timeout

        if (tcsetattr (fd, TCSANOW, &tty) != 0){
        	printf("error %d setting term attributes", errno);
		}
}


void parse_position(char *ptr){
	int items_read;
   	char field [ 100 ];
   	int n;
    while (*ptr != '\0') {
    	items_read = sscanf(ptr, "%31[^,]%n", field, &n);
    	printf("field = \"%s\"\n", field);
    	field[0]='\0';
    	if (items_read == 1)
    		ptr += n; /* advance the pointer by the number of characters read */
   		if ( *ptr != ',' ) {
   		 	break; /* didn't find an expected delimiter, done? */
    	}
    	++ptr; /* skip the delimiter */
    }
}

int find(char *ptr, int start ,char target,char *label){
	int items_read;
   	char field [ 100 ];
   	int n;

	items_read = 0;
	ptr += start;
    while (*ptr != '\0') {
		if(target == ':'){
    		items_read = sscanf(ptr, "%31[^:]%n", field, &n);
		}
		if(target == ','){
    		items_read = sscanf(ptr, "%31[^,]%n", field, &n);
		}
		if(target == ']'){
    		items_read = sscanf(ptr, "%31[^]]%n", field, &n);
		}
		if(strcmp(label, "none")!=0){
			printf("%s = %s \n",label,field);
		}
    	if (items_read == 1)
    		return n+1;
    }
	return -1; // didn't find an expected delimiter, done?
}

int main(int argc, char *argv[]){
	char *portname = "/dev/ttyS0";
	char buf [100];
	int fd = open (portname, O_RDWR | O_NOCTTY | O_SYNC);
	int next;
	int n;

	if (fd < 0){
        printf("error %d opening ", errno);
		printf("%s: %s",  portname, strerror (errno));
        return errno;
	}

	set_interface_attribs (fd, B115200, 0); // 115,200 bps, 
											// 8n1 (no parity)

//	printf("Input string[%d]: %s\n",strlen(argv[1]), argv[1]);
	write (fd, argv[1], strlen(argv[1]));
	if((strlen(argv[1]))!=1){
		write (fd, "\n", 1); 
		usleep ((strlen(argv[1])+25) * 100);   	// wait for uart
	}else{
		usleep (10000);   					// wait for position
	}
	
	

	if(strcmp(argv[1], "$$")==0){
	int i;
		for(i=0;i<2000;i++){	
			n=read (fd, buf, 1);  		// read results
			if(buf[0] != 0x0d){
				printf("%c",buf[0]);
			}
			if(n==0){break;};
		}
	}


	n=read (fd, buf, sizeof buf);  		// read results
	if(n==4){
		if((buf[0] == 'o') && (buf[1] == 'k')){
			return 1;
		}else{
			printf("Result: %s\n",buf);
			return -1;
		}
	}

//	printf("Characters received = %d text:%s\n",n,buf);

//	buf[0] = '+';//hack because php doesn't like <>
//parse_position(buf);

if(strcmp(argv[1],"?")==0){
	next = 0;
	next +=find(buf,next,':',"none");
	next +=find(buf,next,',',"X ");
	next +=find(buf,next,',',"Y ");
	next +=find(buf,next,',',"Z ");
}

if(strcmp(argv[1], "$i")==0){
	next = 0;
	next +=find(buf,next,']',"Version");
}


//	printf("%s\n",buf);
	return 0;
}
